@@ -1290,7 +1290,7 @@ long uwsgi_num_from_file(char *filename, int quiet) {
 // setup for a new request
 void wsgi_req_setup(struct wsgi_request *wsgi_req, int async_id, struct uwsgi_socket *uwsgi_sock) {
 
-	wsgi_req->app_id = uwsgi.default_app;
+	wsgi_req->app_id = -1;
 
 	wsgi_req->async_id = async_id;
 	wsgi_req->sendfile_fd = -1;
@@ -1901,33 +1901,6 @@ void uwsgi_unix_signal(int signum, void (*func) (int)) {
 	}
 }
 
-char *uwsgi_get_exported_opt(char *key) {
-
-	int i;
-
-	for (i = 0; i < uwsgi.exported_opts_cnt; i++) {
-		if (!strcmp(uwsgi.exported_opts[i]->key, key)) {
-			return uwsgi.exported_opts[i]->value;
-		}
-	}
-
-	return NULL;
-}
-
-char *uwsgi_get_optname_by_index(int index) {
-
-	struct uwsgi_option *op = uwsgi.options;
-
-	while (op->name) {
-		if (op->shortcut == index) {
-			return op->name;
-		}
-		op++;
-	}
-
-	return NULL;
-}
-
 int uwsgi_list_has_num(char *list, int num) {
 
 	char *list2 = uwsgi_concat2(list, "");
@@ -3380,8 +3353,10 @@ struct uwsgi_app *uwsgi_add_app(int id, uint8_t modifier1, char *mountpoint, int
 		}
 	}
 
-	if ((mountpoint_len == 0 || (mountpoint_len = -1 && mountpoint[0] == '/')) && uwsgi.default_app == -1) {
-		uwsgi.default_app = id;
+	if (!uwsgi.no_default_app) {
+		if ((mountpoint_len == 0 || (mountpoint_len = -1 && mountpoint[0] == '/')) && uwsgi.default_app == -1) {
+			uwsgi.default_app = id;
+		}
 	}
 
 	return wi;
@@ -3779,26 +3754,6 @@ error:
 	return NULL;
 }
 
-// evaluate a math expression
-#ifdef UWSGI_MATHEVAL
-double uwsgi_matheval(char *expr) {
-#ifdef UWSGI_DEBUG
-	uwsgi_log("matheval expr = %s\n", expr);
-#endif
-	double ret = 0.0;
-	void *e = evaluator_create(expr);
-	if (!e)
-		return ret;
-	ret = evaluator_evaluate(e, 0, NULL, NULL);
-	evaluator_destroy(e);
-	return ret;
-}
-char *uwsgi_matheval_str(char *expr) {
-	double ret = uwsgi_matheval(expr);
-	return uwsgi_num2str((int) ret);
-}
-#endif
-
 int uwsgi_kvlist_parse(char *src, size_t len, char list_separator, char kv_separator, ...) {
 	size_t i;
 	va_list ap;
