@@ -324,7 +324,7 @@ XS(XS_call) {
 	dXSARGS;
 
         char *func;
-        uint16_t size = 0;
+        uint64_t size = 0;
         int i;
         char *argv[256];
         uint16_t argvs[256];
@@ -351,6 +351,41 @@ XS(XS_call) {
 	XSRETURN_UNDEF;
 }
 
+XS(XS_rpc) {
+
+        dXSARGS;
+
+	char *node;
+        char *func;
+        uint64_t size = 0;
+        int i;
+        char *argv[256];
+        uint16_t argvs[256];
+        STRLEN arg_len;
+
+        psgi_check_args(2);
+
+	node = SvPV_nolen(ST(0));
+        func = SvPV_nolen(ST(1));
+
+        for(i=0;i<(items-2);i++) {
+                argv[i] = SvPV(ST(i+2), arg_len);
+                argvs[i] = arg_len;
+        }
+
+        // response must be always freed
+        char *response = uwsgi_do_rpc(node, func, items-2, argv, argvs, &size);
+        if (response) {
+                ST(0) = newSVpv(response, size);
+                sv_2mortal(ST(0));
+                free(response);
+                XSRETURN(1);
+        }
+
+        XSRETURN_UNDEF;
+}
+
+
 
 XS(XS_suspend) {
 
@@ -404,9 +439,20 @@ XS(XS_i_am_the_lord) {
 	}
         XSRETURN_NO;
 }
-
 #endif
 
+XS(XS_connection_fd) {
+	dXSARGS;
+
+	psgi_check_args(0);	
+
+	struct wsgi_request *wsgi_req = current_wsgi_req();
+
+	ST(0) = newSViv(wsgi_req->fd);
+        sv_2mortal(ST(0));
+        XSRETURN(1);	
+}
+
 XS(XS_websocket_handshake) {
 
 	dXSARGS;
@@ -622,6 +668,45 @@ XS(XS_metric_get) {
         XSRETURN(1);
 }
 
+XS(XS_chunked_read) {
+	dXSARGS;
+        int timeout = 0;
+	size_t len = 0;
+
+	psgi_check_args(0);
+	if (items > 0) {
+        	timeout = SvIV(ST(0));
+        }
+        struct wsgi_request *wsgi_req = current_wsgi_req();
+        char *chunk = uwsgi_chunked_read(wsgi_req, &len, timeout, 0);
+        if (!chunk) {
+		croak("unable to receive chunked part");
+		XSRETURN_UNDEF;
+        }
+
+	ST(0) = newSVpv(chunk, len);
+        sv_2mortal(ST(0));
+        XSRETURN(1);
+}
+
+XS(XS_chunked_read_nb) {
+        dXSARGS;
+        size_t len = 0;
+
+        psgi_check_args(0);
+
+        struct wsgi_request *wsgi_req = current_wsgi_req();
+        char *chunk = uwsgi_chunked_read(wsgi_req, &len, 0, 1);
+        if (!chunk) {
+		if (uwsgi_is_again()) XSRETURN_UNDEF;
+                croak("unable to receive chunked part");
+                XSRETURN_UNDEF;
+        }
+
+        ST(0) = newSVpv(chunk, len);
+        sv_2mortal(ST(0));
+        XSRETURN(1);
+}
 
 void init_perl_embedded_module() {
 	psgi_xs(reload);
@@ -633,6 +718,7 @@ void init_perl_embedded_module() {
 	psgi_xs(cache_clear);
 
 	psgi_xs(call);
+	psgi_xs(rpc);
 	psgi_xs(wait_fd_read);
 	psgi_xs(wait_fd_write);
 	psgi_xs(async_sleep);
@@ -646,6 +732,9 @@ void init_perl_embedded_module() {
 #ifdef UWSGI_SSL
 	psgi_xs(i_am_the_lord);
 #endif
+
+	psgi_xs(connection_fd);
+
 	psgi_xs(alarm);
 	psgi_xs(websocket_handshake);
 	psgi_xs(websocket_recv);
@@ -665,5 +754,8 @@ void init_perl_embedded_module() {
 	psgi_xs(metric_div);
 	psgi_xs(metric_get);
 	psgi_xs(metric_set);
+
+	psgi_xs(chunked_read);
+	psgi_xs(chunked_read_nb);
 }
 
