@@ -141,6 +141,11 @@ static VALUE rack_uwsgi_i_am_the_lord(VALUE *class, VALUE legion_name) {
 }
 #endif
 
+static VALUE rack_uwsgi_connection_fd(VALUE *class) {
+	struct wsgi_request *wsgi_req = current_wsgi_req();
+        return INT2NUM(wsgi_req->fd);
+}
+
 
 
 static VALUE rack_uwsgi_setprocname(VALUE *class, VALUE rbname) {
@@ -800,7 +805,7 @@ static VALUE uwsgi_ruby_signal_registered(VALUE *class, VALUE signum) {
 static VALUE uwsgi_ruby_do_rpc(int argc, VALUE *rpc_argv, VALUE *class) {
 
 	char *node = NULL, *func;
-        uint16_t size = 0;
+        uint64_t size = 0;
 
         char *argv[256];
         uint16_t argvs[256];
@@ -1166,6 +1171,8 @@ void uwsgi_rack_init_api() {
 #ifdef UWSGI_SSL
 	uwsgi_rack_api("i_am_the_lord", rack_uwsgi_i_am_the_lord, 1);
 #endif
+
+	uwsgi_rack_api("connection_fd", rack_uwsgi_connection_fd, 0);
 	
 
         uwsgi_rack_api("cache_get", rack_uwsgi_cache_get, -1);
